#include "DirectXLib.h"
//#define INITGUID
//#include <d3d12.h>


#pragma comment(lib, "d3d12.lib")
#pragma comment(lib, "dxgi.lib")


DirectX12::DirectX12(void)
	: m_RtvDescriptorSize(0)
	, m_FrameIndex(0)
	, m_FenceEvent(nullptr)
	, m_Width(0)
	, m_Height(0)
{}

DirectX12::~DirectX12(void)
{}


bool DirectX12::Initialize(HWND hWnd, int width, int height, bool isFullscreen)
{
	if (m_pDevice != nullptr) return false;

	UINT dxgiFactoryFlags = 0;
#if defined(_DEBUG)
	{
		ComPtr<ID3D12Debug> debugController;
		if (SUCCEEDED(D3D12GetDebugInterface(IID_PPV_ARGS(&debugController))))
		{
			debugController->EnableDebugLayer();

			dxgiFactoryFlags |= DXGI_CREATE_FACTORY_DEBUG;
		}
	}
#endif

	ComPtr<IDXGIFactory6> pFactory;
	// DXGIFactory1̐
	if (FAILED(CreateDXGIFactory2(dxgiFactoryFlags, IID_PPV_ARGS(&pFactory))))
	{
		return false;
	}
	ComPtr<ID3D12Device> pDevice;
	// D3D12foCX̐
	if (FAILED(D3D12CreateDevice(nullptr, D3D_FEATURE_LEVEL_12_1, IID_PPV_ARGS(&pDevice))))
	{
		return false;
	}

	D3D12_COMMAND_QUEUE_DESC queueDesc{};
	queueDesc.Flags = D3D12_COMMAND_QUEUE_FLAG_NONE;
	queueDesc.Type = D3D12_COMMAND_LIST_TYPE_DIRECT;
	ComPtr<ID3D12CommandQueue> pCommandQueue;
	if (FAILED(pDevice->CreateCommandQueue(&queueDesc, IID_PPV_ARGS(&pCommandQueue))))
	{
		return false;
	}
	ComPtr<ID3D12CommandAllocator> pCommandAllocator;
	if (FAILED(pDevice->CreateCommandAllocator(D3D12_COMMAND_LIST_TYPE_DIRECT, IID_PPV_ARGS(&pCommandAllocator))))
	{
		return false;
	}

	ComPtr<ID3D12GraphicsCommandList> pCommandList;
	if (FAILED(pDevice->CreateCommandList(0, D3D12_COMMAND_LIST_TYPE_DIRECT, pCommandAllocator.Get(), nullptr, IID_PPV_ARGS(&pCommandList))))
	{
		return false;
	}
	// R}h̎tԂȂ̂ŕĂ
	if (FAILED(pCommandList->Close()))
	{
		return false;
	}

	ComPtr<ID3D12Fence> pFence;
	HANDLE FenceEvent;
	// IuWFNg̍쐬
	{
		if (FAILED(pDevice->CreateFence(0, D3D12_FENCE_FLAG_NONE, IID_PPV_ARGS(&pFence))))
		{
			return false;
		}
		// Create an event handle to use for frame synchronization.
		FenceEvent = CreateEvent(nullptr, FALSE, FALSE, nullptr);
		if (FenceEvent == nullptr && FAILED(GetLastError()))
		{
			return false;
		}
	}

	DXGI_SWAP_CHAIN_DESC1 swapChainDesc{};
	swapChainDesc.BufferCount = FrameCount;
	swapChainDesc.Width = width;
	swapChainDesc.Height = height;
	swapChainDesc.Format = DXGI_FORMAT_R8G8B8A8_UNORM;
	swapChainDesc.BufferUsage = DXGI_USAGE_RENDER_TARGET_OUTPUT;
	swapChainDesc.SwapEffect = DXGI_SWAP_EFFECT_FLIP_DISCARD;
	swapChainDesc.SampleDesc.Count = 1;
	// SwapChain̍쐬
	ComPtr<IDXGISwapChain1> pSwapChain;
	if (FAILED(pFactory->CreateSwapChainForHwnd(
		pCommandQueue.Get(),
		hWnd,
		&swapChainDesc,
		nullptr,
		nullptr,
		&pSwapChain
	)))
	{
		return false;
	}

	m_pDevice = pDevice;
	pSwapChain.As(&m_pSwapChain);
	m_pCommandQueue = pCommandQueue;
	m_pCommandAllocator = pCommandAllocator;
	m_pCommandList = pCommandList;
	m_FrameIndex = m_pSwapChain->GetCurrentBackBufferIndex();
	m_FenceEvent = FenceEvent;
	m_pFence = pFence;
	m_Width = width;
	m_Height = height;

	return true;
}

// DirectX̏I
void DirectX12::Finalize(void)
{
	if (m_FenceEvent)
	{
		CloseHandle(m_FenceEvent);
		m_FenceEvent = nullptr;
	}
	m_pDevice.Reset();
	m_pSwapChain.Reset();
	m_pCommandQueue.Reset();
	m_pCommandAllocator.Reset();
	m_pCommandList.Reset();
	m_pFence.Reset();
}

